define("de_epages/amazonpayments/checkoutconfirmation", [
  "jquery",
  "ep/ui/form",
  "ep/fn/busy"
], function ($) {
  var basketData = $("#addressBookWidgetDiv").data(),
    BasketId = basketData.basketid,
    ReferenceId = basketData.refid,
    MerchantId = basketData.merchantid;

  function getParameterValue(parameterName) {
    var result = null,
        tmp = [];
    var items = location.search.substr(1).split("&");
    for (var index = 0; index < items.length; index++) {
        tmp = items[index].split("=");
        if (tmp[0] === parameterName) result = decodeURIComponent(tmp[1]);
    }
    return result;
  }

  function injectAmazonWidgets() {
    new OffAmazonPayments.Widgets.AddressBook({
      sellerId: MerchantId,
      amazonOrderReferenceId: ReferenceId,
      displayMode: "Read",
      design: {size: {width: "290px", height: "185px"}},
      onError: function (error) {
        handleError(error.getErrorCode(), "AddressError");
      }
    }).bind("addressBookWidgetDiv");

    new OffAmazonPayments.Widgets.Wallet({
      sellerId: MerchantId,
      amazonOrderReferenceId: ReferenceId,
      displayMode: "Read",
      design: {size: {width: "290px", height: "185px"}},
      onError: function (error) {
        handleError(error.getErrorCode(), "WalletError");
      }
    }).bind("walletWidgetDiv");
  }

  function recoverMarkedCheckboxes() {
      if ($('#AcceptTAC').length) { $('#AcceptTAC').prop("checked", true); }
      if ($('#AcceptCustomerInformation').length) { $('#AcceptCustomerInformation').prop("checked", true); }

      if (getParameterValue("RatingInvitationFlag") === "true") {
          $('#IsRatingInvitationAllowed').prop("checked", true);
      }
  }

  function handleAmazonCallback() {
    if ("Success" === getParameterValue("AuthenticationStatus")
        && "SaveMultiCheckoutConfirmation" === getParameterValue("ChangeAction")) {
        recoverMarkedCheckboxes();
        document.forms["BasketForm"].submit();
    } else {
      injectAmazonWidgets();
      $('button.SubmitButton').on('click.submitAmazonWidget', function(event) {
        event.stopPropagation();
        event.preventDefault();
        ep('#BasketForm').busy('show');
        onBuyButtonClick();
        return false;
      });
      $(document.body).busy('hide');
    }
  }

  function handleError(errorCode, errorType) {
    window.location.href = "?ViewAction=TriggerAmazonPaymentsError&ObjectID=" + BasketId + "&" + errorType + "=" + errorCode;
  }

  function onBuyButtonClick() {
    var validationResult = validateRequiredCheckboxes();
    if (validationResult) {
      new OffAmazonPayments.initConfirmationFlow(MerchantId, ReferenceId, function(confirmationFlow) {
        confirmOrderReference(confirmationFlow);
      });
    } else {
      ep('#BasketForm').busy('hide');
    }
  }

  function validateRequiredCheckboxes() {
    var basketForm = ep("#BasketForm");

    basketForm.trigger("validate");
    return basketForm.find(".ui-invalid").length === 0;
  }

  function confirmOrderReference(confirmationFlow) {
    var requiredCheckboxes = JSON.stringify({
        TermsAndConditionsFlag: $('#AcceptTAC').prop("checked"),
        CustomerInformationFlag: $('#AcceptCustomerInformation').prop("checked")
    });
    var optionalCheckboxes = JSON.stringify({
        RatingInvitationFlag: $('#IsRatingInvitationAllowed').prop("checked")
    });

    var customerCommentText = $('textarea#CustomerComment').val();

    $.ajax({
        url: '?',
        method: 'POST',
        dataType: 'json',
        data: {
          ChangeAction: 'OrderConfirmationAmazonPayment',
          ObjectID: BasketId,
          RequiredCheckboxes: requiredCheckboxes,
          OptionalCheckboxes: optionalCheckboxes,
          CustomerComment: customerCommentText
        },
        success: function (data) {
          confirmationFlow.success(); // continue Amazon Pay initConfirmationFlow
        },
        error: function (data) { // called on AJAX error and timeout
          confirmationFlow.error(); // abort Amazon Pay initConfirmationFlow
          $('#BasketForm').busy('hide');
          $('#initConfirmationFlowErrorDiv').show();
        },
      timeout: "9000"
    });
  }

  $(document.body).busy("show");
  handleAmazonCallback();

});
